#include "embeddedInfoCanvas.h"

embeddedInfoCanvas::embeddedInfoCanvas(const char* name, const TGWindow* p, UInt_t w, UInt_t h, UInt_t options)
{
	setup();
	canvas = new TRootEmbeddedCanvas(name, p, 970,600);	

	(canvas->GetCanvas())->cd();
	mainPad = new TPad("main","mainT", 0, 0.05,  1,   1);
	info    = new TPaveText(0,0,1,0.05);
	info->AddText("Some basic info.");
  info->SetTextColor(15);
  info->SetBorderSize(0);
  info->SetFillColor(0);

	mainPad->SetFillColor(0);
	mainPad->Draw();
	info->Draw();
}

embeddedInfoCanvas::~embeddedInfoCanvas()
{
	delete[] info;
	delete[] mainPad;
	delete[] canvas;
}

void embeddedInfoCanvas::Draw()
{
	(canvas->GetCanvas())->cd();
	mainPad->Draw();	
	info->Draw();
}

void embeddedInfoCanvas::Paint()
{
	mainPad->Paint();	
}

void embeddedInfoCanvas::Update()
{
  (canvas->GetCanvas())->Update();
	mainPad->Update();	
}

void embeddedInfoCanvas::setText(const char* text)
{
  (canvas->GetCanvas())->cd();
	info->Clear();
	info->AddText(text);
  info->Paint();
  Update();
}

TPad* embeddedInfoCanvas::GetMain()
{
	return mainPad;
}

TCanvas* embeddedInfoCanvas::GetTCanvas()
{
  return canvas->GetCanvas();
}

TRootEmbeddedCanvas* embeddedInfoCanvas::GetEmbedded()
{
	return canvas;
}

void embeddedInfoCanvas::setup()
{
  
  // Borders
  gStyle->SetCanvasBorderMode(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetPadBorderMode(0);
  gStyle->SetDrawBorder(0);
  gStyle->SetCanvasBorderSize(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetStatBorderSize(0);
  gStyle->SetTitleBorderSize(0);

  //Stat
  gStyle->SetOptStat("");
  gStyle->SetStatColor(19);
  gStyle->SetStatTextColor(13);
  gStyle->SetStatFont(42);

  //Label & Axis & Grid
  gStyle->SetLabelFont(42,"xyz");
  gStyle->SetLabelColor(1,"xyz");
  gStyle->SetAxisColor(19,"xyz");
  gStyle->SetNdivisions(1,"xyz");
  gStyle->SetPadTickX(1);
  gStyle->SetPadTickY(1);
  gStyle->SetTickLength(0.02,"xyz");
  gStyle->SetPadGridX(1);
  gStyle->SetPadGridY(1);
  gStyle->SetGridStyle(3);
  gStyle->SetGridColor(18);

  //Title
  gStyle->SetTitleColor(13);
  gStyle->SetTitleFillColor(0);
  gStyle->SetTitleTextColor(13);
  gStyle->SetTitleFont(42,"xyz");
  gStyle->SetTitleFont(42,"h");
  gStyle->SetTitleX(0.10); // spot where histogram title goes
  gStyle->SetTitleW(0.78); // width computed so that title is centered
  gStyle->SetTitleSize(0.06,"xyz");

  // // Say it in black and white!
  gStyle->SetCanvasColor(0);
  gStyle->SetPadColor(0);
  gStyle->SetFrameFillColor(0);
  gStyle->SetFrameLineColor(13);
  gStyle->SetHistFillColor(9);
  gStyle->SetHistLineColor(9);
 
  //Margins
  gStyle->SetPadTopMargin(0.06);
  gStyle->SetPadBottomMargin(0.18);
  gStyle->SetPadLeftMargin(0.1);
  gStyle->SetPadRightMargin(0.15);
  
  //Misc.
  gStyle->SetLineColor(38);
  gStyle->SetFuncWidth(2);
  gStyle->SetFuncColor(46);
  gStyle->SetStripDecimals(kFALSE); // if we have 1.5 do not set 1.0 -> 1
  gStyle->SetTimeOffset(0); // As it should be...
  
}


