#include "fullConfig.h"

int fullConfig::count = 0;

fullConfig::fullConfig(const TGWindow *p, UInt_t h, UInt_t w, masterController* masterControl, UInt_t options) : TGMainFrame(p, h, w, options)
{	
	fullConfig::count++;

	master = masterControl;
	copyMachine( &(master->getCurrentMachine()), &localMachine );
	copyTomography( &(master->getCurrentTomography()), &localTomography );
	SetCleanup( kDeepCleanup );
	
	Connect( "CloseWindow()", "fullConfig", this, "CloseWindow()" );

	addMachineFrame();
	addTomographyFrame();

	//Map Window
	SetWindowName( "Configuration" );
 	
	MapSubwindows();
	Layout();
	Resize( 500, 500 );
	MapWindow();
}

fullConfig::~fullConfig()
{
	;
}

void fullConfig::addMachineFrame()
{
	//Outer most frame. 
	TGCompositeFrame* mFrame = new TGCompositeFrame( this, 350, 20, kVerticalFrame );
	this->AddFrame( mFrame, new TGLayoutHints( kLHintsCenterY | kLHintsExpandX , 1, 1, 1, 1 ) );

  	//Option Boxes and Number Entries
  	TGCompositeFrame* mOptionsFrame = new TGCompositeFrame( mFrame, 350, 20, kVerticalFrame );
	mFrame->AddFrame( mOptionsFrame, new TGLayoutHints( kLHintsTop | kLHintsCenterX, 1, 1, 1, 10 ) );

  	TGCompositeFrame* mOptionsFrame_h       = new TGCompositeFrame( mOptionsFrame, 350, 20, kHorizontalFrame );
	TGCompositeFrame* mOptionsFrame_RF_f    = new TGCompositeFrame( mOptionsFrame, 350, 20, kHorizontalFrame );
	TGCompositeFrame* mOptionsFrame_RF_V    = new TGCompositeFrame( mOptionsFrame, 350, 20, kHorizontalFrame );
	TGCompositeFrame* mOptionsFrame_gammaT  = new TGCompositeFrame( mOptionsFrame, 350, 20, kHorizontalFrame );
	TGCompositeFrame* mOptionsFrame_synPhi  = new TGCompositeFrame( mOptionsFrame, 350, 20, kHorizontalFrame );

	mOptionsFrame->AddFrame( mOptionsFrame_h     , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	mOptionsFrame->AddFrame( mOptionsFrame_RF_f  , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	mOptionsFrame->AddFrame( mOptionsFrame_RF_V  , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	mOptionsFrame->AddFrame( mOptionsFrame_gammaT, new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	mOptionsFrame->AddFrame( mOptionsFrame_synPhi, new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );

	TGLabel* mOptionsLabel_h      = new TGLabel( mOptionsFrame_h     , "Harmonic Number        " );
	TGLabel* mOptionsLabel_RF_f   = new TGLabel( mOptionsFrame_RF_f  , "RF Freq. [Hz]          " );
	TGLabel* mOptionsLabel_RF_V   = new TGLabel( mOptionsFrame_RF_V  , "RF Volt [V]            " );
	TGLabel* mOptionsLabel_gammaT = new TGLabel( mOptionsFrame_gammaT, "GammaT                 " );
	TGLabel* mOptionsLabel_synPhi = new TGLabel( mOptionsFrame_synPhi, "Synchronous Phase [rad]" );

	machineBoxes.resize(0);
	machineBoxes.push_back( new TGNumberEntry( mOptionsFrame_h     , localMachine.h     , 10, -1, TGNumberEntry::kNESInteger , TGNumberEntry::kNEAPositive, TGNumberEntry::kNELNoLimits ) );
	machineBoxes.push_back( new TGNumberEntry( mOptionsFrame_RF_f  , localMachine.RF_f  , 10, -1, TGNumberEntry::kNESReal    , TGNumberEntry::kNEAPositive, TGNumberEntry::kNELNoLimits ) );
	machineBoxes.push_back( new TGNumberEntry( mOptionsFrame_RF_V  , localMachine.RF_V  , 10, -1, TGNumberEntry::kNESReal    , TGNumberEntry::kNEAPositive, TGNumberEntry::kNELNoLimits ) );
	machineBoxes.push_back( new TGNumberEntry( mOptionsFrame_gammaT, localMachine.gammaT, 10, -1, TGNumberEntry::kNESReal    , TGNumberEntry::kNEAPositive, TGNumberEntry::kNELNoLimits ) );
	machineBoxes.push_back( new TGNumberEntry( mOptionsFrame_synPhi, localMachine.synPhi, 10, -1, TGNumberEntry::kNESReal    , TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELNoLimits) );

	mOptionsFrame_h     ->AddFrame( mOptionsLabel_h     , new TGLayoutHints( kLHintsTop | kLHintsLeft , 1, 1, 1, 1 ) );
	mOptionsFrame_RF_f  ->AddFrame( mOptionsLabel_RF_f  , new TGLayoutHints( kLHintsTop | kLHintsLeft , 1, 1, 1, 1 ) );
	mOptionsFrame_RF_V  ->AddFrame( mOptionsLabel_RF_V  , new TGLayoutHints( kLHintsTop | kLHintsLeft , 1, 1, 1, 1 ) );
	mOptionsFrame_gammaT->AddFrame( mOptionsLabel_gammaT, new TGLayoutHints( kLHintsTop | kLHintsLeft , 1, 1, 1, 1 ) );
	mOptionsFrame_synPhi->AddFrame( mOptionsLabel_synPhi, new TGLayoutHints( kLHintsTop | kLHintsLeft , 1, 1, 1, 1 ) );

	mOptionsFrame_h     ->AddFrame( machineBoxes[0], new TGLayoutHints( kLHintsTop | kLHintsRight , 1, 1, 1, 1 ) );
	mOptionsFrame_RF_f  ->AddFrame( machineBoxes[1], new TGLayoutHints( kLHintsTop | kLHintsRight , 1, 1, 1, 1 ) );
	mOptionsFrame_RF_V  ->AddFrame( machineBoxes[2], new TGLayoutHints( kLHintsTop | kLHintsRight , 1, 1, 1, 1 ) );
	mOptionsFrame_gammaT->AddFrame( machineBoxes[3], new TGLayoutHints( kLHintsTop | kLHintsRight , 1, 1, 1, 1 ) );
	mOptionsFrame_synPhi->AddFrame( machineBoxes[4], new TGLayoutHints( kLHintsTop | kLHintsRight , 1, 1, 1, 1 ) );

	//Set, Load, Save Buttons
	TGCompositeFrame* mButtonsFrame = new TGCompositeFrame( mFrame, 350, 20, kHorizontalFrame );
	mFrame->AddFrame( mButtonsFrame, new TGLayoutHints( kLHintsTop | kLHintsCenterX, 1, 1, 1, 10 ) );

	TGTextButton *mSave        = new TGTextButton( mButtonsFrame, "Save Config." , -1 );
  	TGTextButton *mLoad        = new TGTextButton( mButtonsFrame, "Load Config." , -1 );
  	TGTextButton *mSet         = new TGTextButton( mButtonsFrame, "Set Config. " , -1 );
  	TGTextButton *mApply       = new TGTextButton( mButtonsFrame, "Apply To All" , -1 );
  	TGTextButton *mMakeDefault = new TGTextButton( mButtonsFrame, "Make Default" , -1 );

	mSave ->Connect( "Clicked()", "fullConfig", this, "emitButton(=fullConfig::kButton_mSave)" );
	mLoad ->Connect( "Clicked()", "fullConfig", this, "emitButton(=fullConfig::kButton_mLoad)" );
	mSet  ->Connect( "Clicked()", "fullConfig", this, "setMachine()" );
	mApply->Connect( "Clicked()", "fullConfig", this, "setMachineThenEmit(=fullConfig::kButton_mApply)" );
	mMakeDefault->Connect( "Clicked()", "fullConfig", this, "setMachineThenEmit(=fullConfig::kButton_mMakeDefault)" );

  	mButtonsFrame->AddFrame( mSave  , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	mButtonsFrame->AddFrame( mLoad  , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	mButtonsFrame->AddFrame( mSet   , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	mButtonsFrame->AddFrame( mApply , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	mButtonsFrame->AddFrame( mMakeDefault , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	//End Set, Load, Save 
}

void fullConfig::addTomographyFrame()
{
	//Outer most frame.
	TGCompositeFrame* tFrame = new TGCompositeFrame( this, 350, 20, kVerticalFrame );
	this->AddFrame( tFrame, new TGLayoutHints( kLHintsCenterY | kLHintsExpandX , 1, 1, 1, 1 ) );

  	//Option Boxes and Number Entries
	TGCompositeFrame* tOptionsFrame = new TGCompositeFrame( tFrame, 350, 20, kVerticalFrame );
	tFrame->AddFrame( tOptionsFrame, new TGLayoutHints( kLHintsTop | kLHintsCenterX, 1, 1, 1, 10 ) );

  	TGCompositeFrame* tOptionsFrame_MaxIterations = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_ConvLimit     = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_SqrtPPP       = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_Frames        = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_reconFrame    = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_Algorithm     = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_DFCARTDamping = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_pixelsPhi     = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );
	TGCompositeFrame* tOptionsFrame_pixelsdE      = new TGCompositeFrame( tOptionsFrame, 100, 20, kHorizontalFrame );

	tOptionsFrame->AddFrame( tOptionsFrame_MaxIterations, new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_ConvLimit    , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_SqrtPPP      , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_Frames       , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_reconFrame   , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_Algorithm    , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_DFCARTDamping, new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_pixelsPhi    , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );
	tOptionsFrame->AddFrame( tOptionsFrame_pixelsdE     , new TGLayoutHints( kLHintsTop | kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1 ) );

	TGLabel* tOptionsLabel_MaxIterations = new TGLabel( tOptionsFrame_MaxIterations, "Max Iterations " );
	TGLabel* tOptionsLabel_ConvLimit     = new TGLabel( tOptionsFrame_ConvLimit    , "Conv. Limit    " );
	TGLabel* tOptionsLabel_SqrtPPP       = new TGLabel( tOptionsFrame_SqrtPPP      , "Sqrt Part/Pix  " );
	TGLabel* tOptionsLabel_Frames        = new TGLabel( tOptionsFrame_Frames       , "Frames         " );
	TGLabel* tOptionsLabel_reconFrame    = new TGLabel( tOptionsFrame_reconFrame   , "Recon. Frame(!)" );
	TGLabel* tOptionsLabel_Algorithm     = new TGLabel( tOptionsFrame_Algorithm    , "Algorithm      " );
	TGLabel* tOptionsLabel_DFCARTDamping = new TGLabel( tOptionsFrame_DFCARTDamping, "DFCART Damping " );
	TGLabel* tOptionsLabel_pixelsPhi     = new TGLabel( tOptionsFrame_pixelsPhi    , "Pixels Phi     " );
	TGLabel* tOptionsLabel_pixelsdE      = new TGLabel( tOptionsFrame_pixelsdE     , "Pixels dE      " );


	tomographyBoxes.resize(0);
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_MaxIterations, localTomography.MaxIterations      , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAPositive , TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_ConvLimit    , localTomography.ConvergenceLimit   , 10, -1, TGNumberEntry::kNESReal   , TGNumberEntry::kNEAPositive , TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_SqrtPPP      , localTomography.SqrtPPP            , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAPositive , TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_Frames       , localTomography.Frames             , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_reconFrame   , localTomography.ReconstructionFrame, 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELNoLimits ) );
	//tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_Algorithm    , localTomography.algorithm          , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELLimitMinMax,0,3 ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_DFCARTDamping, localTomography.dampingFactor      , 10, -1, TGNumberEntry::kNESReal   , TGNumberEntry::kNEAPositive , TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_pixelsPhi    , localTomography.pixelsPhi          , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELNoLimits ) );
	tomographyBoxes.push_back( new TGNumberEntry( tOptionsFrame_pixelsdE     , localTomography.pixelsdE           , 10, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAAnyNumber, TGNumberEntry::kNELNoLimits ) );

	algorithmSelector = new TGComboBox( tOptionsFrame_Algorithm );
	algorithmSelector->SetWidth( 105 );
	algorithmSelector->AddEntry(" P. ART(Best)", kPROBART_ID  );
	algorithmSelector->AddEntry(" P. ART2     ", kPROBART2_ID );
	algorithmSelector->AddEntry(" P. ART3     ", kPROBART3_ID );
	algorithmSelector->AddEntry(" FCART       ", kFCART_ID    );
	algorithmSelector->AddEntry(" FCART2      ", kFCART2_ID   );
	algorithmSelector->AddEntry(" FCART3      ", kFCART3_ID   );
	algorithmSelector->AddEntry(" DFCART      ", kFCART_ID    );
	algorithmSelector->AddEntry(" TEST        ", kTESTING_ID  );
	algorithmSelector->Select( kPROBART_ID );

	tOptionsFrame_MaxIterations->AddFrame( tOptionsLabel_MaxIterations, new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_ConvLimit    ->AddFrame( tOptionsLabel_ConvLimit    , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_SqrtPPP      ->AddFrame( tOptionsLabel_SqrtPPP      , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_Frames       ->AddFrame( tOptionsLabel_Frames       , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_reconFrame   ->AddFrame( tOptionsLabel_reconFrame   , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_Algorithm    ->AddFrame( tOptionsLabel_Algorithm    , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_DFCARTDamping->AddFrame( tOptionsLabel_DFCARTDamping, new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_pixelsPhi    ->AddFrame( tOptionsLabel_pixelsPhi    , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );
	tOptionsFrame_pixelsdE     ->AddFrame( tOptionsLabel_pixelsdE     , new TGLayoutHints( kLHintsTop | kLHintsLeft, 1, 1, 1, 1 ) );

	tOptionsFrame_MaxIterations->AddFrame( tomographyBoxes[0], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_ConvLimit    ->AddFrame( tomographyBoxes[1], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_SqrtPPP      ->AddFrame( tomographyBoxes[2], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_Frames       ->AddFrame( tomographyBoxes[3], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_reconFrame   ->AddFrame( tomographyBoxes[4], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	// tOptionsFrame_Algorithm    ->AddFrame( tomographyBoxes[5], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_Algorithm    ->AddFrame( algorithmSelector , new TGLayoutHints( kLHintsTop | kLHintsRight | kLHintsExpandY , 1, 0, 0, 0 ) );
	tOptionsFrame_DFCARTDamping->AddFrame( tomographyBoxes[5], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_pixelsPhi    ->AddFrame( tomographyBoxes[6], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );
	tOptionsFrame_pixelsdE     ->AddFrame( tomographyBoxes[7], new TGLayoutHints( kLHintsTop | kLHintsRight, 1, 1, 1, 1 ) );

	//Set, Load, Save
	TGCompositeFrame* tButtonsFrame = new TGCompositeFrame( tFrame, 350, 20, kHorizontalFrame );
	tFrame->AddFrame( tButtonsFrame, new TGLayoutHints( kLHintsTop | kLHintsCenterX, 1, 1, 1, 10 ) );

	TGTextButton *tSave        = new TGTextButton( tButtonsFrame, "Save Config.", -1 );
  	TGTextButton *tLoad        = new TGTextButton( tButtonsFrame, "Load Config.", -1 );
  	TGTextButton *tSet         = new TGTextButton( tButtonsFrame, "Set Config. ", -1 );
  	TGTextButton *tApply       = new TGTextButton( tButtonsFrame, "Apply To All", -1 );
  	TGTextButton *tMakeDefault = new TGTextButton( tButtonsFrame, "Make Default", -1 );

  	tSave ->Connect( "Clicked()", "fullConfig", this, "emitButton(=fullConfig::kButton_tSave)" );
	tLoad ->Connect( "Clicked()", "fullConfig", this, "emitButton(=fullConfig::kButton_tLoad)" );
	tSet  ->Connect( "Clicked()", "fullConfig", this, "setTomography() "  );
	tApply->Connect( "Clicked()", "fullConfig", this, "setTomographyThenEmit(=fullConfig::kButton_tApply)" );
	tMakeDefault->Connect( "Clicked()", "fullConfig", this, "setTomographyThenEmit(=fullConfig::kButton_tMakeDefault)");

  	tButtonsFrame->AddFrame( tSave , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	tButtonsFrame->AddFrame( tLoad , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	tButtonsFrame->AddFrame( tSet  , new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	tButtonsFrame->AddFrame( tApply, new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );
  	tButtonsFrame->AddFrame( tMakeDefault, new TGLayoutHints( kLHintsTop , 1 , 1, 1, 1 ) );

  	//End Set, Load, Save
}

void fullConfig::CloseWindow()
{
	fullConfig::count--;
	delete this;
}

void fullConfig::setTomography()
{
	localTomography.MaxIterations       = tomographyBoxes[0]->GetIntNumber();
	localTomography.ConvergenceLimit    = tomographyBoxes[1]->GetNumber();
	localTomography.SqrtPPP             = tomographyBoxes[2]->GetIntNumber();
	localTomography.Frames              = tomographyBoxes[3]->GetIntNumber();
	localTomography.ReconstructionFrame = tomographyBoxes[4]->GetIntNumber();
	localTomography.algorithm           = algorithmSelector->GetSelected(); 
	localTomography.dampingFactor       = tomographyBoxes[5]->GetNumber();
	localTomography.pixelsPhi           = tomographyBoxes[6]->GetIntNumber();
	localTomography.pixelsdE            = tomographyBoxes[7]->GetIntNumber();

	master->setCurrentTomography( localTomography );
}

void fullConfig::setMachine()
{
	localMachine.h      = machineBoxes[0]->GetIntNumber();
	localMachine.RF_f   = machineBoxes[1]->GetNumber();
	localMachine.RF_V   = machineBoxes[2]->GetNumber();
	localMachine.gammaT = machineBoxes[3]->GetNumber();
	localMachine.synPhi = machineBoxes[4]->GetNumber();

	master->setCurrentMachine( localMachine );
}

void fullConfig::setMachineThenEmit(int button)
{
	setMachine();
	emitButton(button);
}

void fullConfig::setTomographyThenEmit(int button)
{
	setTomography();
	emitButton(button);	
}

Machine_Config fullConfig::getMachine()
{
	return localMachine;
}

Tomography_Config fullConfig::getTomography()
{
	return localTomography;
}

