#include "mainView.h"

mainView::mainView() : TQObject()
{
  viewFormat   = NULL;
  dataControl  = NULL;
  mvPad        = NULL;

  isOld    = true;
}

mainView::mainView( TVirtualPad* pad, std::vector<plotFormat>* pf, masterController* mc ) : TQObject()
{ 
  master       = mc  ;
  viewFormat   = pf  ;
  dataControl  = NULL;
  mvPad        = pad ;

  isOld    = true;

  setStyle();
  buildView(); // build view does NOT update the view. Must call update.
}

mainView::mainView( TVirtualPad* pad, std::vector<plotFormat>* pf, dataController* dc, masterController* mc ) : TQObject()
{ 
  master       = mc  ;
  viewFormat   = pf  ;
  dataControl  = dc  ;
  mvPad        = pad ;

  isOld    = true;

  setStyle();
  buildView(); // build view does NOT update the view. Must call update.
}

mainView::~mainView()
{	
  delete this;
}

void mainView::buildView()
{
  // Build just populates the view. It does not fill
  // histograms or print them. Update will get data to 
  // fill histograms and update the screen. 

  // This will need to be updated to draw the appropriate 
  // plot type to the pad.
  
  cleanView();

  // Should only add to the appropriate plot type vector, TH2F, TH1F TGraph etc. 
  if( hasPad() )
  {
    for( int i = 0 ; i < viewFormat->size(); ++i )
    {
      double xMin = (*viewFormat)[i].padXMin;
      double yMin = (*viewFormat)[i].padYMin;
      double xMax = (*viewFormat)[i].padXMax;
      double yMax = (*viewFormat)[i].padYMax;

      plotPads.push_back( new TPad("name", "name", xMin, yMin, xMax, yMax) );
      plotPads[i]->SetBit( kMustCleanup, kTRUE ); //Makes sure pad is recursively removed. 

      switch( (*viewFormat)[i].plotType )
      {
        case kImage:
          histVector.push_back( new TH2F() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
      
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(16);
      
        break;
        case kData:
        case kProj:
          histVector.push_back( new TH2F() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
                
        break;

        case kEmittanceHisto:
          histVector.push_back( new TH2F() );
          mvPad->cd();
          
          plotPads[i]->Draw();
          plotPads[i]->cd();
    
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(16);
        break;
        case kEmittance:
          multiGraphVector.push_back( new TMultiGraph() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
        break;

        case kDiscrepancy:
          hist1Vector.push_back( new TH1F() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
        break;

        case kDiscrepancyGraph:
          multiGraphVector.push_back( new TMultiGraph() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
        break;

        case kMomentPhase:
          hist1Vector.push_back( new TH1F() );
          mvPad->cd();
          
          plotPads[i]->Draw();
          plotPads[i]->cd();
    
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(16);
        break;
        case kMomentPhaseFFT:
          hist1Vector.push_back( new TH1F() );
          mvPad->cd();
          
          plotPads[i]->Draw();
          plotPads[i]->cd();
    
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(16);
        break;
        case kMachineStatsView:
          TPaveVector.push_back( new TPaveText() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
        break;
        case kReconStatsView:
          TPaveVector.push_back( new TPaveText() );

          mvPad->cd();
          plotPads[i]->Draw();
          plotPads[i]->cd();
        break;
      }//switch
    }//for
  }//if( hasPad() )
}

void mainView::update(dataController* data)
{
  dataControl = data;
  update();
}

void mainView::update()
{
  if( hasNoData() || hasNoPad() )
  {
    ;//Do nothing. 
  }else if( dataControl->isNotEmpty() && hasPad() && viewIsOld() ){

    std::vector< TH2F* >::iterator th2_it;
    std::vector< TH1F* >::iterator th1_it;
    std::vector< TPaveText* >::iterator tpave_it;
    std::vector< TMultiGraph* >::iterator tmgr_it; 
    
    //Only need an iterator if build push_back'ed that type's vector.
    if( histVector.size()       != 0 ) th2_it   = histVector.begin();
    if( hist1Vector.size()      != 0 ) th1_it   = hist1Vector.begin();
    if( TPaveVector.size()      != 0 ) tpave_it = TPaveVector.begin();
    if( multiGraphVector.size() != 0 ) tmgr_it  = multiGraphVector.begin();
    
    for( int i = 0 ; i < viewFormat->size(); ++i )
    {
      double xMin = (*viewFormat)[i].padXMin ;
      double yMin = (*viewFormat)[i].padYMin ;
      double xMax = (*viewFormat)[i].padXMax ;
      double yMax = (*viewFormat)[i].padYMax ;

      int type = (*viewFormat)[i].plotType ;

      int start  = (*viewFormat)[i].startBunch    ;
      int end    = (*viewFormat)[i].endBunch      ;
      int range  = (*viewFormat)[i].bunchesPerRow ;
      int mOrder = (*viewFormat)[i].momentOrder   ;

      plotPads[i]->cd();

      switch( type )
      {
        case kImage:
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(16);

          (*th2_it)->SetOption((*th2_it)->GetOption());
          dataControl->fillRangeHistogram( start, range, (*th2_it), type );

          (*th2_it)->SetMinimum(-1E-300); //Using this && COL CONT1 allows colors down to zero.
          (*th2_it)->Draw("COL CONT1");
          if( (*viewFormat)[i].plotSeparatrix ) dataControl->drawRangeContours_Phi(3.141592, plotPads[i], range ); //Separatrix
          for( int c = 0; c < (*viewFormat)[i].numberOfContours; ++c)
          {
            dataControl->drawRangeContours_MeV(  (*viewFormat)[i].contours[c]  , plotPads[i], range ); //8 MeV curve. this should be settable.
          }
          ++th2_it;
        break;
        case kProj:
        case kData:
          dataControl->fillRangeHistogram( start, range, (*th2_it), type );
          (*th2_it)->Draw("COL");
          ++th2_it;
        break;
        case kEmittanceHisto:
        {
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(18);
          dataControl->fillEmittanceHisto( start, range, (*th2_it) );
          TH2F* forContours = (TH2F*)(*th2_it)->Clone();

          Double_t contLev[3] = { 0.33, 0.66, 0.95 };
          forContours->SetContour( 3, &contLev[0] );

          (*th2_it)->Draw("COLZ");
          forContours->Draw("CONT3 SAME");

          ++th2_it;
        }
        break;
        case kEmittance:
        {
          if(*tmgr_it != NULL ) delete (*tmgr_it);
          //It would be better here to fill the MultiGraph at build time
          //Then iteratate over the graphs clearing and refilling as needed.
          //If a hard rebuild is called TMultigraph will get deleted.
          (*tmgr_it) = new TMultiGraph();
          dataControl->fillEmittanceGraph( start, range, (*tmgr_it) );
          (*tmgr_it)->Draw("ac");

          ++tmgr_it;
        }
        break;

        case kDiscrepancy:
        { 
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(kAzure);
          dataControl->fillDiscrepancyHisto( start, range, (*th1_it) );
          (*th1_it)->Draw();

          ++th1_it;
        }
        break;
        case kDiscrepancyGraph:
        { 
          if(*tmgr_it != NULL ) delete (*tmgr_it);

          (*tmgr_it) = new TMultiGraph();
          dataControl->fillDiscrepancyGraph( start, range, (*tmgr_it) );
          (*tmgr_it)->Draw("ac");

          ++tmgr_it;
        }
        break;

        case kMomentPhase:
        {
          //Should have a few more options in this one maybe. 
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(kAzure + mOrder);
        
          dataControl->fillImageMomentPhaseHistogram( mOrder, (*th1_it) );
          (*th1_it)->Draw();

          ++th1_it;
        }
        break;
        case kMomentPhaseFFT:
        {
          //Should have a few more options in this one maybe. 
          gStyle->SetNumberContours(20);
          gStyle->SetHistLineColor(kOrange + mOrder);
          
          dataControl->fillImageMomentPhaseFFTHistogram( mOrder, (*th1_it) );
          (*th1_it)->Draw();

          ++th1_it;
        }
        break;
        case kReconStatsView:
        {
          Tomography_Config tomography = ( dataControl == NULL ) ? master->getCurrentTomography() : dataControl->getTomography();
    
          TPaveText* pave = new TPaveText(0.15,0.15,0.85,0.85);
          pave->SetBit(kMustCleanup, kTRUE);
          pave->Draw();
          reconStatsToPave(pave, tomography);
        }
        break;
        case kMachineStatsView:
        {
          Machine_Config machine = ( dataControl == NULL ) ? master->getCurrentMachine() : dataControl->getMachine();
          
          TPaveText* mpave = new TPaveText(0.15,0.15,0.85,0.85);
          mpave->SetBit(kMustCleanup, kTRUE);
          mpave->Draw();
          machineStatsToPave(mpave, machine);
        }
        break;
        default:
        break;
      }
      mvPad->cd();
      plotPads[i]->Modified();
    }

    mvPad->Update();
  }

  isOld = false;
}

void mainView::updatePreview()
{
  if( hasPad() && viewIsOld() )
  {
    char title[16];
    for( int i = 0 ; i < viewFormat->size(); ++i )
    {
      plotPads[i]->cd();

      int type = (*viewFormat)[i].plotType;

      sprintf(title, "%d", i);
      TText *padTitle = new TText( 0.94, 0.94, title );
      padTitle->SetTextColor(46);
      plotPads[i]->SetFillColor( (kAzure + i % 10) - 9);

      std::stringstream testImage( TARDIS_Resources );
      testImage.seekp( 0, testImage.end );

      switch( type )
      {
        case kReconStatsView:
        {
          Tomography_Config tomography = ( dataControl == NULL ) ? master->getCurrentTomography() : dataControl->getTomography();
    
          TPaveText* pave = new TPaveText(0.15,0.15,0.85,0.85);
          pave->SetBit(kMustCleanup, kTRUE);
          pave->Draw();
          reconStatsToPave(pave, tomography);
        }
        break;
        case kMachineStatsView:
        {
          Machine_Config machine = ( dataControl == NULL ) ? master->getCurrentMachine() : dataControl->getMachine();
          
          TPaveText* mpave = new TPaveText(0.15,0.15,0.85,0.85);
          mpave->SetBit(kMustCleanup, kTRUE);
          mpave->Draw();
          machineStatsToPave(mpave, machine);
        }
        break;
        case kImage:
        {
          testImage << "/testImage.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        case kMomentPhase:
        {
          testImage << "/momentPreview.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        case kMomentPhaseFFT:
        {
          
          testImage << "/momentPreview.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        case kProj:
        case kData:
        {
          testImage << "/projectionPreview.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        case kEmittanceHisto:
        case kEmittance:
        {
          testImage << "/emittancePreview.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        case kDiscrepancy:
        case kDiscrepancyGraph:
        {
          testImage << "/discrepancyPreview.png";
          TImage* test = TImage::Open( testImage.str().c_str() );
          test->SetBit(kMustCleanup, kTRUE);
          test->SetConstRatio(kFALSE);
          test->Draw();
        }
        break;
        default:
        break;
      }
      padTitle->Draw();
      mvPad->cd();
      plotPads[i]->Modified();
    }
    mvPad->Update();
  }
  isOld = false;
}

void mainView::cleanView()
{

  if( hasPad() ){
    if( plotPads.size() > 0 )
    {
      for(int i = 0; i < plotPads.size(); ++i) mvPad->RecursiveRemove(plotPads[i]);
    }
    mvPad->Clear();
  }
  
  histVector.clear();
  hist1Vector.clear();
  TPaveVector.clear();
  multiGraphVector.clear();

  plotPads.clear();

  isOld = true;
}

void mainView::updateFormatPadPositions()
{
  for(int i = 0; i < plotPads.size(); ++i )
  {
    plotPads[i]->GetPadPar(((*viewFormat)[i].padXMin),((*viewFormat)[i].padYMin),((*viewFormat)[i].padXMax),((*viewFormat)[i].padYMax));
  }
}

void mainView::machineStatsToPave(TPaveText* pave, Machine_Config machine)
{
  char line[100];
  pave->SetShadowColor(0);
  pave->SetBorderSize(0);
  pave->SetTextColor(16);
  pave->SetTextAlign(11);
  pave->SetFillColor(0);

  pave->AddText("Machine:");

  sprintf( line, "h  = %d", machine.h);
  pave->AddText(line);
  sprintf( line, "f_{RF}[MHz]  = %6.4f", machine.RF_f*1e-6);
  pave->AddText(line);
  sprintf( line, "V_{RF}[kV]   = %4.2f", machine.RF_V*1e-3);
  pave->AddText(line);
  sprintf( line, "#gamma_{T}  = %4.3f", machine.gammaT);
  pave->AddText( line );
}

void mainView::reconStatsToPave(TPaveText* pave, Tomography_Config tomography)
{
  char line[100];
  pave->SetShadowColor(0);
  pave->SetBorderSize(0);
  pave->SetTextColor(16);
  pave->SetTextAlign(11);
  pave->SetFillColor(0);

  pave->AddText("Reconstruction:");

  sprintf( line, "MaxIterations = %d", tomography.MaxIterations );
  pave->AddText(line);
  sprintf( line, "SqrtPPP = %d", tomography.SqrtPPP );
  pave->AddText(line);
  sprintf( line, "Frames  = %d", tomography.Frames );
  pave->AddText(line);
  sprintf( line, "Reconstruction Frame  = %d", tomography.ReconstructionFrame );
  pave->AddText(line);
  sprintf( line, "Convergence Limit = %3.2e", tomography.ConvergenceLimit);
  pave->AddText( line );
  sprintf( line, "DFCART damping  = %2.1f", tomography.dampingFactor);
  pave->AddText( line );
}

void mainView::setStyle()
{

  TGaxis::SetMaxDigits(2);
  gStyle->SetNdivisions(1,"xy");

  gStyle->SetCanvasBorderMode(0);
  gStyle->SetFrameBorderMode(0);
  gStyle->SetPadBorderMode(0);
  gStyle->SetDrawBorder(0);
  gStyle->SetCanvasBorderSize(0);
  gStyle->SetFrameBorderSize(0);
  gStyle->SetPadBorderSize(0);
  gStyle->SetStatBorderSize(0);
  gStyle->SetTitleBorderSize(0);

  //Stat
  gStyle->SetOptStat("");
  //gStyle->SetOptStat("mr");
  gStyle->SetStatColor(19);
  gStyle->SetStatTextColor(13);
  gStyle->SetStatFont(42);

  //Label & Axis & Grid
  gStyle->SetLabelFont(42,"xyz");
  gStyle->SetLabelColor(13,"xyz");
  gStyle->SetAxisColor(19,"xyz");
  gStyle->SetNdivisions(1,"xyz");
  gStyle->SetPadTickX(1);
  gStyle->SetPadTickY(1);
  gStyle->SetTickLength(0.02,"xyz");
  gStyle->SetPadGridX(1);
  gStyle->SetPadGridY(1);
  gStyle->SetGridStyle(3);
  gStyle->SetGridColor(18);

  //Title
  gStyle->SetTitleColor(13);
  gStyle->SetTitleFillColor(0);
  gStyle->SetTitleTextColor(13);
  gStyle->SetTitleFont(42,"xyz");
  gStyle->SetTitleFont(42,"h");
  gStyle->SetTitleX(0.10); // spot where histogram title goes
  gStyle->SetTitleW(0.78); // width computed so that title is centered
  gStyle->SetTitleSize(0.06,"xyz");

  // Say it in black and white!
  gStyle->SetCanvasColor(0);
  gStyle->SetPadColor(0);
  gStyle->SetFrameFillColor(0);
  gStyle->SetFrameLineColor(0);
  gStyle->SetHistFillColor(9);
  gStyle->SetHistLineColor(9);

  //Margins
  gStyle->SetPadTopMargin(0.1);
  gStyle->SetPadBottomMargin(0.1);
  gStyle->SetPadLeftMargin(0.1);
  gStyle->SetPadRightMargin(0.1);

  //Misc.
  gStyle->SetLineColor(38);
  gStyle->SetFuncWidth(2);
  gStyle->SetFuncColor(46);
  gStyle->SetStripDecimals(kFALSE); // if we have 1.5 do not set 1.0 -> 1
  gStyle->SetTimeOffset(0);         // As it should be...
}
