#ifndef _MAINVIEW_H
#define _MAINVIEW_H

#include <sstream>
#include <string>

//Root includes
#include <TExec.h>
#include <TMultiGraph.h>
#include <TPaveText.h>
#include <TGraph.h>
#include <TCanvas.h>
#include <TGraph.h>
#include <TStyle.h>
#include <TGaxis.h>
#include <TImage.h>
#include <TH2.h>
#include <TH1.h>

//NJE includes
#include "dataController.h"
#include "masterController.h"
#include "Global_Values_Extern.h"
#include "TomographyStructs.h"
//#include "TomographyStructDefaults.h"
#include "TQObject.h"

class TQObject;
class TH2F;
class TCanvas;
class TPaveText;
class dataController;
class masterController;

class mainView : public TQObject
{

	private:
		bool isOld; 

		dataController        *dataControl;
		masterController      *master;

		//Vectors containing plot pointers
		std::vector< TH2F* >   histVector;
		std::vector< TH1F* >   hist1Vector;
		std::vector< TMultiGraph* > multiGraphVector;
		std::vector< TPaveText*   > TPaveVector;

		//Format vector
		std::vector<plotFormat> *viewFormat;
		std::vector<plotFormat> currentFormat; //This will be in use. In case updating doesn't happen.

		//Main Pad and sub-pad vector 
		TVirtualPad *mvPad;
		std::vector< TPad* > plotPads;

		void setStyle();
		void cleanView();

	public:
		
		mainView();
		mainView( TVirtualPad* pad, std::vector<plotFormat>* pf, masterController* mc );
		mainView( TVirtualPad* pad, std::vector<plotFormat>* pf, dataController* dc, masterController* mc );
		~mainView();
		void buildView();

		bool hasPad(){ return (mvPad != NULL); };
		bool hasNoPad(){ return (mvPad == NULL); };
		bool hasData(){ return (dataControl != NULL); };
		bool hasNoData(){ return (dataControl == NULL); };

		void machineStatsToPave(TPaveText* pave, Machine_Config machine      );
		void reconStatsToPave(TPaveText* pave  , Tomography_Config tomography);
		void update(dataController* data);	
		void update();	
		void updatePreview();	
		void updateFormatPadPositions();

		bool viewIsOld(){ return isOld; };
		bool viewIsNotOld(){ return !isOld; };

		void setViewIsOld(){ isOld = true; };

		void setPad(TVirtualPad* pad){ mvPad = pad; };
		void disconnectPad(){ mvPad = NULL; };


	ClassDef(mainView, 0);
};

#endif
