#include "mainWindow.h" //Class Header

int mainWindow::count = 0;

mainWindow::mainWindow(const TGWindow *p, int numberOfTabs, UInt_t w, UInt_t h ) : tabbedInfoWindow(p,w,h)
{
  mainWindow::count++;

  whichAlgorithm = kMenu_Algo_FCART;         //no damping, no pix < 0, etc. is default

  //Menubar with Pop-ups_______________________________________________

  fMenuBar = new TGMenuBar( this, 100, 20, kHorizontalFrame );
  TGLayoutHints *fMenuPopupLayout = new TGLayoutHints( kLHintsTop | kLHintsLeft );

  TGPopupMenu *fMenu_File  = new TGPopupMenu( gClient->GetDefaultRoot() );
  TGPopupMenu *fMenu_Conf  = new TGPopupMenu( gClient->GetDefaultRoot() );
  TGPopupMenu *fMenu_Input = new TGPopupMenu( gClient->GetDefaultRoot() );
  TGPopupMenu *fMenu_Help  = new TGPopupMenu( gClient->GetDefaultRoot() );

  //'File' Entries
  fMenu_File->AddEntry("Open Preprocessed"      , kMenu_File_Open_PRP       );
  fMenu_File->AddEntry("Open Raw Data"          , kMenu_File_Open_RAW       );
  fMenu_File->AddEntry("Open Ming-Jen"          , kMenu_File_Open_Ming_Jen  );
  fMenu_File->AddEntry("Save Raw Data"          , kMenu_File_Save_Raw_Data  );
  fMenu_File->AddEntry("Save Reconstructions"   , kMenu_File_Save_Recon     );
  fMenu_File->AddEntry("Save Recon Profiles "   , kMenu_File_Save_Recon_Prof);
  fMenu_File->AddEntry("Save Current Tab as PDF", kMenu_File_Save_Canvas_PDF);

  //Input
  fMenu_Conf->AddEntry("Scope Config      ", kMenu_Config_Scope         );
  fMenu_Conf->AddEntry("Recon Parameters  ", kMenu_Config_Recon         );
  fMenu_Conf->AddEntry("Color Schemes     ", kMenu_Config_Format        );
  fMenu_Conf->AddEntry("Plot Layout       ", kMenu_Config_PlotLayout    );
  
  //'Help' Entries
  fMenu_Help->AddEntry("Help?"          , 1 );
  fMenu_Help->AddEntry("Ha!"            , 2 );
  fMenu_Help->AddEntry("Ok, call x4555.", 3 );

  //Connect All the menu drop downs
  fMenu_File->Connect("Activated(Int_t)" , "mainWindow", this, "emitSignal(Int_t)");
  //fMenu_Input->Connect("Activated(Int_t)", "mainWindow", this, "emitSignal(Int_t)");
  fMenu_Conf->Connect("Activated(Int_t)" , "mainWindow", this, "emitSignal(Int_t)");

  fMenuBar->AddPopup("File"      , fMenu_File , fMenuPopupLayout );
  fMenuBar->AddPopup("Config"    , fMenu_Conf , fMenuPopupLayout );
  //fMenuBar->AddPopup("Input"     , fMenu_Input, fMenuPopupLayout );
  fMenuBar->AddPopup("Help"      , fMenu_Help , fMenuPopupLayout );

  AddFrame(fMenuBar, new TGLayoutHints(kLHintsTop | kLHintsCenterX | kLHintsExpandX, 1, 1, 0, 0));

  TGHorizontal3DLine *MenuSep = new TGHorizontal3DLine(this); 
  AddFrame(MenuSep, new TGLayoutHints(kLHintsTop | kLHintsExpandX));

  //Frame for Tabs_____________________________________________
  TGHorizontalFrame *fr1 = new TGHorizontalFrame(this,600,40);
  AddFrame(fr1, new TGLayoutHints(kLHintsExpandX,0,0,1,0));

  // All Buttons
  TGLayoutHints *leftButtons  = new TGLayoutHints(kLHintsLeft, 3, 3, 3, 3);
  TGLayoutHints *rightButtons = new TGLayoutHints(kLHintsRight, 3, 3, 3, 3);

  TGTextButton *bTomo    = new TGTextButton(fr1, " Reconstruct" , kButton_Tomo );
  TGTextButton *bMlti    = new TGTextButton(fr1, " MultiBunch"  , kButton_Mlti );
  TGTextButton *bHist    = new TGTextButton(fr1, " History"     , kButton_Hist );
  TGTextButton *bOneShot = new TGTextButton(fr1, " One Shot "   , kButton_ScopeSingle );

  //Needs to be accessible from setLoopButton
  bLoop     = new TGTextButton(fr1, " Start " , kButton_Loop); 
  bGetScope = new TGTextButton(fr1, " Get Scope ", kButton_getScope );

  //These populate from the left
  fr1->AddFrame( bTomo, leftButtons );
  fr1->AddFrame( bMlti, leftButtons );
  fr1->AddFrame( bHist, leftButtons );

  //These populate from the right.
  fr1->AddFrame( bLoop     , rightButtons );
  fr1->AddFrame( bOneShot  , rightButtons );
  fr1->AddFrame( bGetScope , rightButtons );

  setLoopButton(false);
  setScopeButton(false);
  
  bGetScope->Connect("Clicked()" , "mainWindow", this, "emitSignal(=mainWindow::kButton_getScope)"    );
  bTomo->Connect("Clicked()"     , "mainWindow", this, "emitSignal(=mainWindow::kButton_Tomo)"        );
  bMlti->Connect("Clicked()"     , "mainWindow", this, "emitSignal(=mainWindow::kButton_Mlti)"        );
  bHist->Connect("Clicked()"     , "mainWindow", this, "emitSignal(=mainWindow::kButton_Hist)"        );
  bLoop->Connect("Clicked()"     , "mainWindow", this, "emitSignal(=mainWindow::kButton_Loop)"        );
  bOneShot->Connect("Clicked()"  , "mainWindow", this, "emitSignal(=mainWindow::kButton_ScopeSingle)" );

  //Create Tabs____________________________________________
  
  addTabFrame( numberOfTabs );//from tabbed info window base class

  //Bottom Bar___________________________________________

  TGHorizontalFrame *statusFrame = new TGHorizontalFrame(this, 100, 100);

  //status bar
  Int_t parts[] = {75,25};
  fStatus = new TGStatusBar( statusFrame, 20, 10, kHorizontalFrame );
  fStatus->SetParts( parts, 2 );

  //progress bar
  progress = new TGHProgressBar( statusFrame, TGProgressBar::kStandard, 100 );
  progress->SetFillType(TGProgressBar::kBlockFill);
  progress->SetBarColor(0x0066FF); // Some blue.

  //Add widgets to status frame
  statusFrame->AddFrame( fStatus , new TGLayoutHints( kLHintsExpandX, 1,1,1,1 ) );
  statusFrame->AddFrame( progress, new TGLayoutHints( kLHintsLeft   , 1,20,1,1 ) );

  AddFrame( statusFrame, new TGLayoutHints(kLHintsBottom | kLHintsExpandX, 1, 1, 1, 1 ) );

  fStatus->SetText("Let's Tomographize.");
  //All Widgets have been added________________________________________
  
  //Name may be overridden from masterController. 
  SetWindowName("Tomography And Related Diagnostics In Synchrotrons : Main Injector");
  MapSubwindows();
  Resize( GetDefaultSize() );
  MapWindow();

  //Connections to self
  TQObject::Connect( "CloseWindow()", "mainWindow", this, "closeWindow()");
  TQObject::Connect( "mainWindow"   , "raise()"   , "mainWindow", this, "wantsFocus()" );

  for( int i = 0; i < numberOfTabs; ++i ) emitPad( i, getCanvas(i)->GetMain() );//Allows view to connect to pads. 

}
 
mainWindow::~mainWindow()
{
  Destroyed();
}

void mainWindow::closeWindow()
{
  mainWindow::count--;
}

void mainWindow::setLoopButton(bool isLooping)
{
  if( isLooping )
  {
    bLoop->SetText(" Stop ");
    bLoop->SetBackgroundColor(0xdc143c);
    bLoop->SetTextColor(0xffffff);
  }else{
    bLoop->SetText(" Start ");
    bLoop->SetBackgroundColor(0x4be22b);
    bLoop->SetTextColor(0xffffff);
  }
}

void mainWindow::setScopeButton( bool hasScope )
{
  if( hasScope )
  {
    bGetScope->SetText(" Kill Scope ");
    bGetScope->SetBackgroundColor(0xdc143c);
    bGetScope->SetTextColor(0xffffff);
  }else{
    bGetScope->SetText(" Get Scope ");
    bGetScope->SetBackgroundColor(0xc0c0c0);
    bGetScope->SetTextColor(0x000000);
  }
}

void mainWindow::setStatus( const char* status, const int part )
{
  fStatus->SetText(status, part);
  gSystem->DispatchOneEvent();
};

void mainWindow::setProgressStatus( const char* status )
{
  setStatus(status, 1);
}

void mainWindow::setProgress( const int percent )
{
  progress->SetPosition( (float)percent );
  gSystem->DispatchOneEvent();
}

