#include "plotLayoutWindow.h" 

int plotLayoutWindow::count = 0;

//Begin plotLayoutWindow Functions ___________________________________
plotLayoutWindow::plotLayoutWindow(const TGWindow *p, UInt_t w, UInt_t h, masterController* mc, std::vector<plotFormat> *pf, UInt_t options) : TGMainFrame(p, w, h, options){

  plotLayoutWindow::count++;

  SetCleanup(kDeepCleanup);
  formatToModify = pf;
  thisFormat     = (*pf);
  master         = mc;
  
  Connect("CloseWindow()","plotLayoutWindow",this,"closeWindow()");
  DontCallClose();

  createPreviewPane(); //In this instance this pane must be built first, since it populates the config, or the comboBoxes won't be built right. 
  createSideBar();

  //Map Window 
  SetWindowName("Format Main Window Plots");
  MapSubwindows();
  Resize();
  MapWindow();
  Layout();

}

plotLayoutWindow::~plotLayoutWindow()
{

}

void plotLayoutWindow::closeWindow()
{
  plotLayoutWindow::count--;
  delete this;
}


void plotLayoutWindow::createSideBar()
{
  TGVerticalFrame  *OptionFrame = new TGVerticalFrame( this, 600, 400 );
  OptionFrame->SetMinWidth(400);
  OptionFrame->SetMaxWidth(400);

  std::vector< TGCompositeFrame* > OptionSubFrame;
  std::vector< TGLabel* >          SelectorLabels;

  int maxNumberOfPlots = 10;
  int initialSelection;
  Pixel_t labelColor;
  char labelNumber[16];

  int startBunchDefault;
  int endBunchDefault;

  for(int i = 0; i < maxNumberOfPlots; ++i)
  {
    startBunchDefault = ( i < thisFormat.size() ) ? thisFormat[i].startBunch : 1; //For some reason this is giving garbage.
    endBunchDefault   = ( i < thisFormat.size() ) ? thisFormat[i].endBunch   : 2; //For some reason this is giving garbage.

    OptionSubFrame.push_back( new TGCompositeFrame( OptionFrame,  400, 400, kHorizontalFrame ) ); 

    PlotSelectors.push_back(  new TGComboBox( OptionSubFrame[i] ) );

    minRangeSelectors.push_back( new TGNumberEntry( OptionSubFrame[i], startBunchDefault, 3, -1, TGNumberEntry::kNESInteger ) );
    maxRangeSelectors.push_back( new TGNumberEntry( OptionSubFrame[i], endBunchDefault  , 3, -1, TGNumberEntry::kNESInteger ) );

    sprintf( labelNumber, " %d ", i );
    SelectorLabels.push_back( new TGLabel( OptionSubFrame[i], &labelNumber[0] ) );

    OptionSubFrame[i]->AddFrame(PlotSelectors[i]     , new TGLayoutHints(  kLHintsExpandX | kLHintsExpandY , 2, 2, 2, 2 ) );
    OptionSubFrame[i]->AddFrame(minRangeSelectors[i] , new TGLayoutHints(  kLHintsExpandX | kLHintsExpandY , 2, 2, 2, 2 ) );
    OptionSubFrame[i]->AddFrame(maxRangeSelectors[i] , new TGLayoutHints(  kLHintsExpandX | kLHintsExpandY , 2, 2, 2, 2 ) );
    OptionSubFrame[i]->AddFrame(SelectorLabels[i]    , new TGLayoutHints(  kLHintsNormal                   , 2, 2, 2, 2 ) );

    OptionFrame->AddFrame(OptionSubFrame[i], new TGLayoutHints( kLHintsCenterX | kLHintsExpandX, 2, 2, 2, 2 ) );

    //Can use a single vector of <pair(string, enumValue)> to populate this. Then just iterate over that
    //vector. Will only have to update plots in one place. 
    PlotSelectors[i]->AddEntry("Recon. Image                ", kImage            );
    PlotSelectors[i]->AddEntry("Projection                  ", kProj             );
    PlotSelectors[i]->AddEntry("Input Data                  ", kData             );
    PlotSelectors[i]->AddEntry("Emittance                   ", kEmittance        );
    PlotSelectors[i]->AddEntry("Emittance Histo             ", kEmittanceHisto   );
    PlotSelectors[i]->AddEntry("Machine Stats               ", kMachineStatsView );
    PlotSelectors[i]->AddEntry("Recon Stats                 ", kReconStatsView   );
    PlotSelectors[i]->AddEntry("Moment Phase                ", kMomentPhase      );
    PlotSelectors[i]->AddEntry("Moment Phase FFT(coupling)  ", kMomentPhaseFFT   );
    PlotSelectors[i]->AddEntry("Final Disc.                 ", kDiscrepancy      );
    PlotSelectors[i]->AddEntry("Final Disc. Graph           ", kDiscrepancyGraph );
    
    initialSelection = i < thisFormat.size() ? thisFormat[i].plotType : kImage;
    PlotSelectors[i]->Select( initialSelection );

  }

  comboBoxesEnable();

  TGHorizontalFrame* buttonsMainFrame  = new TGHorizontalFrame( OptionFrame, 600, 400 );
  TGVerticalFrame*   buttonsLeftFrame  = new TGVerticalFrame( buttonsMainFrame, 600, 400 );
  TGVerticalFrame*   buttonsRightFrame = new TGVerticalFrame( buttonsMainFrame, 600, 400 );

  TGTextButton* applyButton    = new TGTextButton( buttonsRightFrame, " Apply Format "  , kButton_Apply  );
  TGTextButton* previewButton  = new TGTextButton( buttonsRightFrame, "Preview Format", kButton_Preview  );
  TGTextButton* addAPlotButton = new TGTextButton( buttonsLeftFrame , " Add A Plot  " , kButton_AddAPlot );
  TGTextButton* remAPlotButton = new TGTextButton( buttonsLeftFrame , "Remove A Plot" , kButton_RemAPlot );

  buttonsMainFrame->AddFrame( buttonsLeftFrame  , new TGLayoutHints( kLHintsLeft  | kLHintsExpandX , 2, 2, 2, 2 ) );
  buttonsMainFrame->AddFrame( buttonsRightFrame , new TGLayoutHints( kLHintsRight | kLHintsExpandX , 2, 2, 2, 2 ) );

  buttonsRightFrame-> AddFrame( applyButton    , new TGLayoutHints( kLHintsCenterX ,1 ,1 ,1 ,1) );
  buttonsRightFrame-> AddFrame( previewButton  , new TGLayoutHints( kLHintsCenterX ,1 ,1 ,1 ,1) );
  buttonsLeftFrame->  AddFrame( addAPlotButton , new TGLayoutHints( kLHintsCenterX ,1 ,1 ,1 ,1) );
  buttonsLeftFrame->  AddFrame( remAPlotButton , new TGLayoutHints( kLHintsCenterX ,1 ,1 ,1 ,1) );

  OptionFrame->AddFrame( buttonsMainFrame, new TGLayoutHints( kLHintsTop | kLHintsRight | kLHintsExpandX , 2, 2, 2, 2 ) );
  AddFrame( OptionFrame, new TGLayoutHints( kLHintsTop | kLHintsRight | kLHintsExpandX , 2, 2, 2, 2 ) );

  applyButton->Connect("Clicked()"   , "plotLayoutWindow" , this  , "applyFormat()");
  previewButton->Connect("Clicked()" , "plotLayoutWindow" , this  , "previewFormat()");

  addAPlotButton->Connect("Clicked()", "plotLayoutWindow" , this  , "changeNumberOfPlots(=plotLayoutWindow::kButton_AddAPlot)");
  remAPlotButton->Connect("Clicked()", "plotLayoutWindow" , this  , "changeNumberOfPlots(=plotLayoutWindow::kButton_RemAPlot)");
}

void plotLayoutWindow::createPreviewPane()
{

  TGVerticalFrame  *CanvasFrame = new TGVerticalFrame( this, 600, 300 );

  PreviewPane = new TRootEmbeddedCanvas( "Preview", CanvasFrame, 600,400 );
  MainCanvas  = PreviewPane->GetCanvas(); 

  CanvasFrame->AddFrame( PreviewPane , new TGLayoutHints( kLHintsExpandX | kLHintsExpandY, 2, 2, 2, 2 ) );
  AddFrame(CanvasFrame, new TGLayoutHints( kLHintsExpandX | kLHintsExpandY , 2, 2, 2, 2 ) );

  MainCanvas->Update();

  buildDefaultView(); //This has to happen before you build the combo box stuff, but the canvas has to exist.  Since thisFormat has no size. 
}

void plotLayoutWindow::changeNumberOfPlots(int pressed)
{
  int currentSize = thisFormat.size();
  bool proceed    = false;

  if( pressed == kButton_AddAPlot && currentSize < PlotSelectors.size() )
  {
    thisFormat.push_back( thisFormat[ 0 ] );
    proceed = true;
  }else if( pressed == kButton_RemAPlot && currentSize > 1 ){
    thisFormat.pop_back();
    proceed = true; 
  }
  currentSize = thisFormat.size();

  if( proceed )
  {
    comboBoxesEnable();
    //This little algorithm makes a grid. 
    //It tries to pack the plots in in a reasonable way.  
    int sizeX = 0;
    int sizeY = 0;
    int tooSmall = true;
    while( tooSmall )
    {
      if( (sizeX+1) * sizeY >= thisFormat.size() )
      { 
        ++sizeX;
        tooSmall = false;
      }else if( (sizeX+1) * (sizeY+1) >= thisFormat.size() ){
        ++sizeX;
        ++sizeY;
        if(sizeX * sizeY - thisFormat.size() >= 2 )
        {
          ++sizeX;
          --sizeY;
        }
        tooSmall = false;
      }else{
        ++sizeX;
        ++sizeY;
      }
    }

    for(int s = 0; s < currentSize; ++s)
    {
      int x =  s % sizeX;
      int y = (s - x)/sizeX;
      thisFormat[s].padXMin = (float)x/(float)sizeX;
      thisFormat[s].padYMin = (float)y/(float)sizeY;
      thisFormat[s].padXMax = (float)(x+1)/(float)sizeX;
      thisFormat[s].padYMax = (float)(y+1)/(float)sizeY;
    }
    thisView->buildView();
    previewFormat();
  }
}

void plotLayoutWindow::comboBoxesEnable()
{
  //or disable as the case may be. 
  Pixel_t color;
  for(int i = 0; i < PlotSelectors.size(); ++i)
  {
    color = i < thisFormat.size() ? 0xffffff : 0x7f7f7f; //White : Gray; 
    PlotSelectors[i]->ChangeSubframesBackground(color);  
    PlotSelectors[i]->Paint();
  }
}

void plotLayoutWindow::applyFormat()
{
  previewFormat();
  thisView->updateFormatPadPositions();
  thisView->buildView();

  int start, end;
  (*formatToModify).clear();

  for( int i = 0 ; i < thisFormat.size(); ++i )
  {
    formatToModify->push_back( thisFormat[i] );

    start = minRangeSelectors[i]->GetIntNumber();
    end   = start < 0 ? start : maxRangeSelectors[i]->GetIntNumber(); //If lower number is less than zero, we're asking for only the average.

    if( (*formatToModify)[i].plotType == kMomentPhase    ) (*formatToModify)[i].momentOrder = minRangeSelectors[i]->GetIntNumber();
    if( (*formatToModify)[i].plotType == kMomentPhaseFFT ) (*formatToModify)[i].momentOrder = minRangeSelectors[i]->GetIntNumber();

    (*formatToModify)[i].startBunch    = start;
    (*formatToModify)[i].endBunch      = end;
    (*formatToModify)[i].bunchesPerRow = end - start <= 0 ? 1 : end - start;
  }

  thisView->setViewIsOld();
  thisView->update();
  viewModified(); //emits signal letting world know viewToModify should be marked as old now.
}

void plotLayoutWindow::buildDefaultView()
{
  
  thisView = new mainView( (TVirtualPad*)MainCanvas, &thisFormat, master );

  thisView->updatePreview();

}

void plotLayoutWindow::previewFormat()
{
  for(int i = 0; i < thisFormat.size(); ++i) thisFormat[i].plotType = PlotSelectors[i]->GetSelected();
  thisView->updatePreview();
}

void plotLayoutWindow::restoreDefault()
{

}
