#include "structInspectorWindow.h"

int structInspectorWindow::count = 0;

// structInspectorWindow::structInspectorWindow(const TGWindow* p, const TGWindow* main, DAQStruct* someStruct, UInt_t w, UInt_t h) : TGTransientFrame(p, main, w, h)
structInspectorWindow::structInspectorWindow(const TGWindow* p, DAQStruct* someStruct, UInt_t w, UInt_t h) : TGMainFrame(p, w, h)
{
	thisStruct = someStruct;
	structInspectorWindow::count++;
	SetCleanup(kDeepCleanup);

	Connect("CloseWindow()", "structInspectorWindow", this, "CloseWindow()");
	DontCallClose();

	TGVerticalFrame*   labelFrame  = new TGVerticalFrame(   this, 100, 100 );
	TGHorizontalFrame* entryFrame  = new TGHorizontalFrame( labelFrame, 100, 100 );
	TGHorizontalFrame* buttonFrame = new TGHorizontalFrame( this, 100, 100 );
	
	AddFrame( labelFrame , new TGLayoutHints(kLHintsTop    | kLHintsCenterX | kLHintsExpandX, 10, 10, 10, 10));
	AddFrame( buttonFrame, new TGLayoutHints(kLHintsBottom | kLHintsCenterX | kLHintsExpandX, 10, 10, 10, 10));

	//Make a text entry for the title.
	labelFrame->AddFrame( entryFrame , new TGLayoutHints(kLHintsTop    | kLHintsCenterX | kLHintsExpandX, 10, 10, 10, 10));

	titleField = new TGTextEntry( entryFrame, &(thisStruct->title[0]) );
	titleField->SetMaxLength(60); //This is the max length of the title field of DAQStruct
	TGLabel* titleLabel = new TGLabel( entryFrame, " Title ");

	entryFrame->AddFrame( titleField, new TGLayoutHints(kLHintsCenterY| kLHintsCenterX | kLHintsExpandX , 1, 1, 1, 1));
	entryFrame->AddFrame( titleLabel, new TGLayoutHints(kLHintsCenterY| kLHintsCenterX, 1, 1, 1, 1));

	char notSettable[128];
	std::vector< TGLabel* > DAQStructLabels;
	std::vector< TGLabel* > DAQTrigLabels;
	std::vector< TGLabel* > DAQScopeLabels;

	//Header Info
	DAQStructLabels.push_back( new TGLabel( labelFrame, "Header Info"  ) );

	sprintf( notSettable, "version = %d", thisStruct->version );
	DAQStructLabels.push_back( new TGLabel( labelFrame, notSettable ) );

	sprintf( notSettable, "time = %s", thisStruct->time );// time
	DAQStructLabels.push_back( new TGLabel( labelFrame, notSettable  ) );

	//Trigger Info
	DAQTrigLabels.push_back( new TGLabel( labelFrame, "Trigger Info"  ) );

	sprintf(notSettable, "numTraces = %d", thisStruct->DAQTrig.numTraces );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable     ) );

	sprintf(notSettable, "turnsDelay = %d", thisStruct->DAQTrig.turnsDelay );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable      ) );

	sprintf(notSettable, "beamSynchOffset = %d", thisStruct->DAQTrig.beamSynchOffset );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable          ) );

	sprintf(notSettable, "trigDelay = %f", thisStruct->DAQTrig.trigDelay  );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable     ) );

	sprintf(notSettable, "trigEvents = %d", thisStruct->DAQTrig.trigEvents[0] );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable         ) );

	sprintf(notSettable, "trigState = %d", thisStruct->DAQTrig.trigState    );
	DAQTrigLabels.push_back( new TGLabel( labelFrame, notSettable        ) );

	//Scope Info
	DAQScopeLabels.push_back( new TGLabel( labelFrame, "Scope Info"  ) );

	sprintf( notSettable, " numFrames = %d" , thisStruct->DAQScope.numFrames  );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " range = %f" , thisStruct->DAQScope.range      );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " level = %f " , thisStruct->DAQScope.level      );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " offset = %f " , thisStruct->DAQScope.offset     );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " channel = %d " , thisStruct->DAQScope.channel    );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " interval = %f " , thisStruct->DAQScope.interval   );
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	sprintf( notSettable, " ptsPerFrame = %d" , thisStruct->DAQScope.ptsPerFrame);
	DAQScopeLabels.push_back( new TGLabel( labelFrame, notSettable) );

	for(int i = 0; i < DAQStructLabels.size(); ++i) labelFrame->AddFrame( DAQStructLabels[i], new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 3, 3, 3, 3) );
	labelFrame->AddFrame( new TGHorizontal3DLine(labelFrame, 200), new TGLayoutHints(kLHintsCenterX | kLHintsExpandX | kLHintsCenterY, 3, 3, 3, 3) );

	for(int i = 0; i < DAQTrigLabels.size(); ++i) labelFrame->AddFrame( DAQTrigLabels[i]  , new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 3, 3, 3, 3) );

	labelFrame->AddFrame( new TGHorizontal3DLine(labelFrame, 200), new TGLayoutHints(kLHintsCenterX | kLHintsExpandX | kLHintsCenterY, 3, 3, 3, 3) );
	for(int i = 0; i < DAQScopeLabels.size(); ++i) labelFrame->AddFrame( DAQScopeLabels[i] , new TGLayoutHints(kLHintsCenterX | kLHintsCenterY, 3, 3, 3, 3) );

	//Button to continue with the save or cancel the save. 
	TGTextButton* proceed = new TGTextButton( buttonFrame, "Continue", -1 );
	TGTextButton* cancel  = new TGTextButton( buttonFrame, "Cancel  ", -1 ); 
	
	proceed->Associate(this);
	cancel ->Associate(this);

	proceed->Connect( "Clicked()", "structInspectorWindow", this, "continueThenClose()" );
	cancel ->Connect( "Clicked()", "structInspectorWindow", this, "CloseWindow()"       );

	buttonFrame->AddFrame( proceed, new TGLayoutHints(kLHintsBottom | kLHintsCenterX, 1, 1, 1, 1) );
	buttonFrame->AddFrame( cancel , new TGLayoutHints(kLHintsBottom | kLHintsCenterX, 1, 1, 1, 1) );

	SetWindowName("Struct Inspector");

  	MapSubwindows();
  	Resize();
  	MapWindow(); 

  	fClient->WaitFor(this); //This window takes over while it's around.
};

structInspectorWindow::~structInspectorWindow()
{
	this->DeleteWindow();
	structInspectorWindow::count--;
}

void structInspectorWindow::updateTitle()
{
	sprintf( (*thisStruct).title, "%s", titleField->GetText() );
}

void structInspectorWindow::continueThenClose()
{
	updateTitle();
	this->CloseWindow();
}
