#include "thumbWindow.h" 

int thumbWindow::count = 0;

//Begin thumbWindow Functions ___________________________________
thumbWindow::thumbWindow(const TGWindow *p, UInt_t w, UInt_t h, masterController* masterControl, std::vector< plotFormat > *pf, UInt_t options): tabbedInfoWindow(p,w,h)
{
    thumbWindow::count++;

    mainPlotFormat = pf;

    master = masterControl; 

    SetCleanup(kDeepCleanup);

    Connect( "CloseWindow()", "thumbWindow", this, "CloseWindow()" );
    DontCallClose();

    whatToPlot = kMenu_Opts_Recon;

    //Menu Bar Setup 
    fMenuBar = new TGMenuBar(this, 100, 20, kHorizontalFrame);
    TGLayoutHints *fMenuPopupLayout = new TGLayoutHints(kLHintsTop | kLHintsLeft );

    TGPopupMenu *fMenu_File  = new TGPopupMenu( gClient->GetDefaultRoot() );
    TGPopupMenu *fMenu_Opts  = new TGPopupMenu( gClient->GetDefaultRoot() );

    fMenu_File->AddEntry("Save Canvas As Image", kMenu_File_Save_Canvas_PDF );

    fMenu_Opts->AddEntry("Recon Only"         , kMenu_Opts_Recon           );
    fMenu_Opts->AddEntry("Recon + Data"       , kMenu_Opts_Recon_Data      );
    fMenu_Opts->AddEntry("Recon + Prof + Data", kMenu_Opts_Recon_Prof_Data );
    //fMenu_Opts->CheckEntry(Menu_Opts_Recon);

    fMenuBar->AddPopup("File"   , fMenu_File , fMenuPopupLayout );
    fMenuBar->AddPopup("Options", fMenu_Opts , fMenuPopupLayout );
    fMenu_Opts->CheckEntry(whatToPlot); //Default value is checked. 
    AddFrame(fMenuBar, new TGLayoutHints(kLHintsTop | kLHintsCenterX | kLHintsExpandX, 1, 1, 0, 0));

    fMenu_Opts->Connect("Activated(Int_t)", "masterController", master, "thumbWindowSignal(Int_t)");
    fMenu_File->Connect("Activated(Int_t)", "masterController", master, "thumbWindowSignal(Int_t)");

    TGHorizontal3DLine *MenuSep = new TGHorizontal3DLine(this); 
    AddFrame(MenuSep, new TGLayoutHints(kLHintsTop | kLHintsExpandX));

    //Sub Frames  
    TGHorizontalFrame *fr1 = new TGHorizontalFrame( this, 1200, 600 );

    TGCompositeFrame *labelAndEntry_1 = new TGCompositeFrame( fr1, 500, 20, kHorizontalFrame );
    TGCompositeFrame *labelAndEntry_2 = new TGCompositeFrame( fr1, 500, 20, kHorizontalFrame );
    TGCompositeFrame *labelAndEntry_3 = new TGCompositeFrame( fr1, 500, 20, kHorizontalFrame );

    //Setup widgets 
    TGTextButton *bUpdate = new TGTextButton( fr1, " Thumbs", kButton_Update);
    //bUpdate->Connect("Clicked()", "masterController", master, "thumbWindowSignal(=thumbWindow::kButton_Update)");
    bUpdate->Connect("Clicked()", "thumbWindow", this, "createUserView()");
    bUpdate->Connect("Clicked()", "thumbWindow", this, "emitButton(=thumbWindow::kButton_Update)");

    BegBox = new TGNumberEntry( labelAndEntry_1,  1, 3, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAPositive, TGNumberEntry::kNELLimitMinMax, 0, 82 );
    EndBox = new TGNumberEntry( labelAndEntry_2, 82, 3, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAPositive, TGNumberEntry::kNELLimitMinMax, 1, 82 );
    RowBox = new TGNumberEntry( labelAndEntry_3, 17, 3, -1, TGNumberEntry::kNESInteger, TGNumberEntry::kNEAPositive, TGNumberEntry::kNELLimitMinMax, 1, 82 );

    TGLabel *label_1 = new TGLabel( labelAndEntry_1, "Start" );
    TGLabel *label_2 = new TGLabel( labelAndEntry_2, "End"   );
    TGLabel *label_3 = new TGLabel( labelAndEntry_3, "Max/row" );

    TGLayoutHints *TabLayout = new TGLayoutHints( kLHintsExpandX | kLHintsExpandY, 0,0,1,0 );
    TGLayoutHints *butLayout = new TGLayoutHints( kLHintsTop | kLHintsLeft, 3,3,3,3        );
    TGLayoutHints *boxLayout = new TGLayoutHints( kLHintsTop | kLHintsLeft, 3,3,3,3        );

    AddFrame( fr1, new TGLayoutHints( kLHintsExpandX | kLHintsLeft, 0,0,1,0) );

    //Create Tabs____________________________________________ 
    
    int InitNumTabs = master->getNumberOfTabs();
    addTabFrame( InitNumTabs );

    //Add all frames and widgets

    labelAndEntry_1->AddFrame( label_1, boxLayout );
    labelAndEntry_2->AddFrame( label_2, boxLayout );
    labelAndEntry_3->AddFrame( label_3, boxLayout );  

    labelAndEntry_1->AddFrame( BegBox, boxLayout );
    labelAndEntry_2->AddFrame( EndBox, boxLayout );
    labelAndEntry_3->AddFrame( RowBox, boxLayout );

    fr1->AddFrame( bUpdate, butLayout );
    fr1->AddFrame( labelAndEntry_1, butLayout );
    fr1->AddFrame( labelAndEntry_2, butLayout );
    fr1->AddFrame( labelAndEntry_3, butLayout );
   
    
    //Map Window
    SetWindowName("Multi-Bunch View");
    MapSubwindows();
    Resize();
    MapWindow();

    TQObject::Connect( "thumbWindow", "raise()", "thumbWindow", this, "wantsFocus()" );
    //Master connections
    TQObject::Connect( "masterController", "makeTabCurrent(int)"    , "thumbWindow", this, "setTab(int)"             );
    TQObject::Connect( "masterController", "updateInfo(int, char*)" , "thumbWindow", this, "setInfo(int, char*)"  );

    for(int i = 0; i < InitNumTabs; ++i) emitPad( i, getCanvas(i)->GetMain() );

}

thumbWindow::~thumbWindow()
{
  ;
}

void thumbWindow::CloseWindow()
{
  thumbWindow::count--;
  TQObject::Disconnect( "thumbWindow", "raise()", this, "wantsFocus()" );
  //Master connections
  TQObject::Disconnect( "masterController", "makeTabCurrent(int)"    , this, "setTab(int)"             ); 
  TQObject::Disconnect( "masterController", "updateInfo(int, char*)" , this, "setInfo(int, char*)"     );
  master->closeThumbWindow();
  delete this;
}

int thumbWindow::getPlotOption()
{
  return whatToPlot;
}

int thumbWindow::getStartBox()
{
  return BegBox->GetIntNumber();
}

int thumbWindow::getEndBox()
{
  return EndBox->GetIntNumber();
}

int thumbWindow::getPerRowBox()
{
  return RowBox->GetIntNumber();
}

void thumbWindow::setPlotOption(Int_t id)
{
  whatToPlot = id;               //Cascade to here if a Menu_Opts 
  fMenuBar->GetPopup("Options")->UnCheckEntries();
  fMenuBar->GetPopup("Options")->CheckEntry(whatToPlot);
}

void thumbWindow::createUserView()
{
  
  int start  = BegBox->GetIntNumber();
  int end    = EndBox->GetIntNumber();
  int perRow = RowBox->GetIntNumber();
  int totalBunches = end > start ? end - start + 1: 1;    //inclusive of start and end
  perRow = totalBunches < perRow ? totalBunches : perRow; //if perRow > num bunches shorten.
  int rows    = 1;
  int rowMult = 1;
  while( rows * perRow < totalBunches ) ++rows;

  switch(whatToPlot)
  {
    case kMenu_Opts_Recon           :
      rowMult = 1;
    break;
    case kMenu_Opts_Recon_Data      :
      rowMult = 2;
    break;
    case kMenu_Opts_Recon_Prof_Data :
      rowMult = 3;
    break;
  }
  rows *= rowMult;
  double step = 1.0/(double)rows;
  std::vector< plotFormat > scratchFormat(rows);
  char status[256];
  int rowStart = start;
  for( int r = 0; r < rows; r += rowMult )
  {
    // need to clip per row if past end
    //perRow = perRow > (end - rowStart) + 1 ? (end - rowStart) + 1 : perRow; 
    sprintf(status, "perRow = %d, rowStart = %d, rows = %d", perRow, rowStart, rows);
    master->statusUpdate(status);
    scratchFormat[r].plotType      = kImage;
    scratchFormat[r].padXMin       = 0.0;
    scratchFormat[r].padYMin       = (rows-r)*step;
    scratchFormat[r].padXMax       = 1.0;
    scratchFormat[r].padYMax       = (rows-r-1)*step;
    scratchFormat[r].startBunch    = rowStart;
    scratchFormat[r].bunchesPerRow = perRow;
    scratchFormat[r].plotSeparatrix = false;

    scratchFormat[r].contours.clear();
    scratchFormat[r].numberOfContours = scratchFormat[r].contours.size();

    if( whatToPlot == kMenu_Opts_Recon_Data || whatToPlot == kMenu_Opts_Recon_Prof_Data )
    {
      scratchFormat[r+1].plotType      = kData;
      scratchFormat[r+1].padXMin       = 0.0;
      scratchFormat[r+1].padYMin       = (rows-(r+1))*step;
      scratchFormat[r+1].padXMax       = 1.0;
      scratchFormat[r+1].padYMax       = (rows-(r+1)-1)*step;
      scratchFormat[r+1].startBunch    = rowStart;
      scratchFormat[r+1].bunchesPerRow = perRow;
      scratchFormat[r+1].plotSeparatrix = false;

      scratchFormat[r+1].contours.clear();
      scratchFormat[r+1].numberOfContours = scratchFormat[r].contours.size();
    }

    if(whatToPlot == kMenu_Opts_Recon_Prof_Data )
    { 
      scratchFormat[r+2].plotType      = kProj;
      scratchFormat[r+2].padXMin       = 0.0;
      scratchFormat[r+2].padYMin       = (rows-(r+2))*step;
      scratchFormat[r+2].padXMax       = 1.0;
      scratchFormat[r+2].padYMax       = (rows-(r+2)-1)*step;
      scratchFormat[r+2].startBunch    = rowStart;
      scratchFormat[r+2].bunchesPerRow = perRow;
      scratchFormat[r+2].plotSeparatrix = false;

      scratchFormat[r+2].contours.clear();
      scratchFormat[r+2].numberOfContours = scratchFormat[r].contours.size();
    }
    rowStart += perRow;
  }

  scratchFormat.swap( (*mainPlotFormat) );
  viewModified();

}
